# Copyright 2021- Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)

from datetime import datetime, timedelta

from odoo import api, fields, models
from odoo.tools import date_utils


class AccEnedisRawCDC(models.Model):
    _name = "acc.enedis.raw.cdc"
    _description = "CDC Enedis brute"
    _inherit = "acc.curves.mixin"

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    name = fields.Char("Libellé du fichier")
    partner_id = fields.Many2one(
        "res.partner",
        "Contact associé",
        ondelete="restrict",
        readonly=True,
    )
    comp_data_type = fields.Selection(
        [
            ("cons", "cons"),
            ("prod", "prod"),
        ],
        string="Type de donnée",
    )
    power = fields.Float("Puissance en W")
    date_slot = fields.Datetime("Date")
    timestep = fields.Integer(string="Pas de temps en minute", default=30)

    # ------------------------------------------------------
    # SQL Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Default methods
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Onchange / Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # CRUD methods (ORM overrides)
    # ------------------------------------------------------
    def init(self):
        self._cr.execute(
            """
            CREATE INDEX IF NOT EXISTS acc_enedis_raw_cdc_operation_type_date_idx
                ON acc_enedis_cdc (acc_operation_id, comp_data_type, date_slot DESC)
                WHERE acc_operation_id IS NOT NULL and comp_data_type IS NOT NULL;
        """
        )

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Business methods
    # ------------------------------------------------------
    @api.model
    def get_repartition_data(self, operation_id):
        """
        on va chercher les courbes du mois
        précédent pour tous les compteurs de l'opération
        On obtient un dictionnaire au format suivant :
        {[date_slot:
            {"conso": { [PRM conso: puissance,] }
            {"prod_totale": puissance}
        ]}
        Par exemple :
        {
            datetime.datetime(2024, 5, 21, 17, 0):
                {'conso': {
                    '50053496845898': 3500.0,
                    '30002321416210': 13333.0,
                    '30002321075221': 1333.0,
                    '30002320232384': 259500.0,
                    '23265122997876': 54.0,
                    '23265122910287': 1294.0,
                    '23225759755250': 424.0,
                    '23216787154523': 1656.0
                },
                'prod_totale': 29333.0
        },
        {
            datetime.datetime(2024, 5, 21, 17, 30):
                {'conso': {
                    '50053496845898': 5500.0,
                    '30002321416210': 16000.0,
                    '30002321075221': 833.0,
                    '30002320232384': 223000.0,
                    '23265122997876': 1264.0,
                    '23265122910287': 1050.0,
                    '23225759755250': 592.0,
                    '23216787154523': 1844.0
                },
                'prod_totale': 23167.0
            },
        }
        """
        data = {}

        # get current date
        current_date = datetime.now()

        start_date, end_date = date_utils.get_month(
            datetime(current_date.year, current_date.month, 1) - timedelta(days=1)
        )
        end_date = end_date + timedelta(days=1)
        prod_per_slot = self.read_group_curves(
            acc_operation_id=operation_id.id,
            start_date=start_date,
            end_date=end_date,
            fields=["power", "date_slot"],
            groupby=["date_slot:minute"],
            values_type=["prod"],
            exclude_empty_partner=False,
        )

        for prod in prod_per_slot:
            # Initialize dict per slot
            slot = fields.Datetime.from_string(
                prod["__range"]["date_slot:minute"]["from"]
            )
            data.update({slot: {}})
            data[slot].update({"conso": {}})
            # Fill prod_totale
            data[slot].update({"prod_totale": prod["power"]})

        # Fill in conso per counter per slot
        conso_per_slot = self.env["acc.enedis.raw.cdc"].search(
            self.get_curves_domain(
                acc_operation_id=operation_id.id,
                start_date=start_date,
                end_date=end_date,
                values_type=["cons"],
                exclude_empty_partner=False,
            )
        )
        for conso in conso_per_slot:
            data[conso.date_slot]["conso"].update(
                {conso.acc_counter_id.name: conso.power}
            )
        return data
